"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createMultipartUploadFactory = exports.MultipartUploadFactory = void 0;
var _clientS = require("@webiny/aws-sdk/client-s3");
var _error = require("@webiny/error");
class MultipartUploadFactory {
  constructor(params) {
    this.client = params.client;
    this.bucket = params.bucket;
    this.filename = params.filename;
    this.createHandler = params.createHandler;
  }
  async start(params) {
    const resumeUploadId = params?.uploadId;
    if (resumeUploadId !== undefined) {
      return this.continue({
        uploadId: resumeUploadId
      });
    }
    const cmd = new _clientS.CreateMultipartUploadCommand({
      Bucket: this.bucket,
      Key: this.filename
    });
    const result = await this.client.send(cmd);
    const uploadId = result.UploadId;
    if (uploadId) {
      return this.createHandler({
        uploadId,
        client: this.client,
        bucket: this.bucket,
        filename: this.filename,
        parts: undefined
      });
    }
    throw new _error.WebinyError({
      message: "Could not initiate multipart upload.",
      code: "S3_ERROR",
      data: {
        bucket: this.bucket,
        filename: this.filename
      }
    });
  }
  async continue(params) {
    const result = await this.listParts({
      uploadId: params.uploadId
    });
    const parts = result.Parts.map(part => {
      if (!part.ETag || part.PartNumber === undefined) {
        return null;
      }
      return {
        tag: part.ETag.replaceAll('"', ""),
        partNumber: part.PartNumber
      };
    }).filter(part => !!part).sort((a, b) => {
      return a.partNumber - b.partNumber;
    });
    return this.createHandler({
      client: this.client,
      bucket: this.bucket,
      filename: this.filename,
      uploadId: params.uploadId,
      parts
    });
  }
  async listParts(params) {
    const cmd = new _clientS.ListPartsCommand({
      Bucket: this.bucket,
      Key: this.filename,
      UploadId: params.uploadId
    });
    let result;
    try {
      result = await this.client.send(cmd);
    } catch (ex) {
      throw new _error.WebinyError({
        message: `Failed to list parts: ${ex.message}`,
        code: "S3_ERROR",
        data: {
          metadata: ex.$metadata,
          bucket: this.bucket,
          filename: this.filename,
          uploadId: params.uploadId
        }
      });
    }
    if (!result.UploadId || !result.Parts?.length) {
      throw new _error.WebinyError({
        message: "Could not find the upload.",
        code: "S3_ERROR",
        data: {
          bucket: this.bucket,
          filename: this.filename,
          uploadId: params.uploadId
        }
      });
    }
    return result;
  }
}
exports.MultipartUploadFactory = MultipartUploadFactory;
const createMultipartUploadFactory = params => {
  return new MultipartUploadFactory(params);
};
exports.createMultipartUploadFactory = createMultipartUploadFactory;

//# sourceMappingURL=MultipartUploadFactory.js.map