"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createMultipartUpload = exports.MultipartUploadHandler = void 0;
var _clientS = require("@webiny/aws-sdk/client-s3");
var _error = require("@webiny/error");
var _bytes = _interopRequireDefault(require("bytes"));
var _MultipartUploadHandlerPauseResult = require("./MultipartUploadHandlerPauseResult");
var _MultipartUploadHandlerAbortResult = require("./MultipartUploadHandlerAbortResult");
var _MultipartUploadHandlerCompleteResult = require("./MultipartUploadHandlerCompleteResult");
var _MultipartUploadHandlerAddResult = require("./MultipartUploadHandlerAddResult");
/**
 * Minimum we can send into the S3 is 5MB.
 * We can modify to have this value bigger if required.
 */
const MIN_BUFFER_SIZE = _bytes.default.parse("5MB");
const getMinBufferSize = minBufferSize => {
  if (!minBufferSize) {
    return MIN_BUFFER_SIZE;
  }
  const size = typeof minBufferSize === "number" ? minBufferSize : _bytes.default.parse(minBufferSize);
  if (size && size >= MIN_BUFFER_SIZE) {
    return size;
  }
  return MIN_BUFFER_SIZE;
};
class MultipartUploadHandler {
  buffer = [];
  bufferLength = 0;
  constructor(params) {
    if (!params.uploadId?.length) {
      throw new _error.WebinyError({
        message: `Missing "uploadId" in the multipart upload handler.`,
        code: "MULTIPART_UPLOAD_ERROR"
      });
    } else if (!params.filename?.length) {
      throw new _error.WebinyError({
        message: `Missing "filename" in the multipart upload handler.`,
        code: "MULTIPART_UPLOAD_ERROR"
      });
    }
    this.uploadId = params.uploadId;
    this.client = params.client;
    this.bucket = params.bucket;
    this.filename = params.filename;
    this.parts = params.parts || [];
    this.minBufferSize = getMinBufferSize(params.minBufferSize);
  }
  getBuffer() {
    return {
      buffer: this.buffer,
      bufferLength: this.bufferLength
    };
  }
  async add(buffer) {
    this.buffer.push(buffer);
    this.bufferLength = this.bufferLength + buffer.length;
    if (this.bufferLength < this.minBufferSize) {
      return (0, _MultipartUploadHandlerAddResult.createMultipartUploadHandlerAddResult)({
        parts: this.parts,
        written: false,
        pause: () => {
          return this.pause();
        }
      });
    }
    const bufferLength = this.bufferLength;
    const body = Buffer.concat(this.buffer, bufferLength);
    this.buffer = [];
    this.bufferLength = 0;
    await this.write({
      body,
      bufferLength
    });
    return (0, _MultipartUploadHandlerAddResult.createMultipartUploadHandlerAddResult)({
      parts: this.parts,
      written: true,
      pause: () => {
        return this.pause();
      }
    });
  }
  async complete() {
    const bufferLength = this.bufferLength;
    const body = Buffer.concat(this.buffer, bufferLength);
    this.buffer = [];
    this.bufferLength = 0;
    await this.write({
      body,
      bufferLength
    });
    if (this.parts.length === 0) {
      throw new _error.WebinyError({
        message: `Failed to complete the upload, no parts were uploaded.`,
        code: "S3_ERROR"
      });
    }
    const cmd = new _clientS.CompleteMultipartUploadCommand({
      UploadId: this.uploadId,
      Bucket: this.bucket,
      Key: this.filename,
      MultipartUpload: {
        Parts: this.parts.map(part => {
          return {
            ETag: part.tag,
            PartNumber: part.partNumber
          };
        })
      }
    });
    const result = await this.client.send(cmd);
    return (0, _MultipartUploadHandlerCompleteResult.createMultipartUploadHandlerCompleteResult)({
      result,
      uploadId: this.uploadId,
      parts: this.parts
    });
  }
  async abort() {
    const cmd = new _clientS.AbortMultipartUploadCommand({
      UploadId: this.uploadId,
      Bucket: this.bucket,
      Key: this.filename
    });
    const result = await this.client.send(cmd);
    return (0, _MultipartUploadHandlerAbortResult.createMultipartUploadHandlerAbortResult)({
      result,
      uploadId: this.uploadId,
      parts: this.parts
    });
  }
  async pause() {
    if (this.bufferLength > 0) {
      throw new _error.WebinyError({
        message: `Failed to pause the upload, buffer was not empty.`,
        code: "S3_ERROR"
      });
    } else if (this.parts.length === 0) {
      throw new _error.WebinyError({
        message: `Failed to pause the upload, no parts were uploaded.`,
        code: "S3_ERROR"
      });
    }
    return (0, _MultipartUploadHandlerPauseResult.createMultipartUploadHandlerPauseResult)({
      uploadId: this.uploadId,
      parts: this.parts
    });
  }
  async write(params) {
    if (params.bufferLength <= 0) {
      return false;
    }
    const nextPart = this.getNextPartNumber();
    const cmd = new _clientS.UploadPartCommand({
      Bucket: this.bucket,
      Key: this.filename,
      UploadId: this.uploadId,
      Body: params.body,
      PartNumber: nextPart
    });
    const result = await this.client.send(cmd);
    if (!result.ETag) {
      throw new _error.WebinyError({
        message: `Failed to upload part: ${nextPart}`,
        code: "S3_ERROR"
      });
    }
    this.parts.push({
      partNumber: nextPart,
      tag: result.ETag.replaceAll('"', "")
    });
    return true;
  }
  getUploadId() {
    return this.uploadId;
  }
  getNextPartNumber() {
    if (this.parts.length === 0) {
      return 1;
    }
    const part = this.parts.at(-1);
    if (!part) {
      return 1;
    }
    return part.partNumber + 1;
  }
}
exports.MultipartUploadHandler = MultipartUploadHandler;
const createMultipartUpload = params => {
  return new MultipartUploadHandler(params);
};
exports.createMultipartUpload = createMultipartUpload;

//# sourceMappingURL=MultipartUploadHandler.js.map